@@ -1,6 +1,6 @@
 ;;; forge-commands.el --- Commands  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2018-2024 Jonas Bernoulli
+;; Copyright (C) 2018-2025 Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
 ;; Maintainer: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
@@ -65,14 +65,15 @@ Takes the pull-request as only argument and must return a directory."
     ("f t" "one topic"      forge-pull-topic)
     ("f n" "notifications"  forge-pull-notifications)]
    ["Create"
-    :if forge--get-repository:tracked?
+    :if (##forge-get-repository :tracked?)
+    ("c d" "discussion"     forge-create-discussion)
     ("c i" "issue"          forge-create-issue)
     ("c p" "pull-request"   forge-create-pullreq)
     ("c u" "pr from issue"  forge-create-pullreq-from-issue)
     ("c f" "fork or remote" forge-fork)]
    [:description (lambda ()
                    (cond
-                    ((forge--get-repository:tracked?) "Actions")
+                    ((forge-get-repository :tracked?) "Actions")
                     ((or (magit-gitdir) (forge-repository-at-point))
                      "Forge does not yet track this repository")
                     ("Not inside a Git repository")))
@@ -83,20 +84,22 @@ Takes the pull-request as only argument and must return a directory."
                             "track some repo"
                           "track this repository"))))
     ("/M" "merge with api" forge-merge
-     :if forge--get-repository:tracked? :level 7)]]
+     :if (##forge-get-repository :tracked?)
+     :level 7)]]
   [forge--lists-group
    ["Visit"
-    :inapt-if-not forge--get-repository:tracked?
+    :inapt-if-not (##forge-get-repository :tracked?)
     ("v t" "topic"          forge-visit-topic)
+    ("v d" "discussion"     forge-visit-discussion)
     ("v i" "issue"          forge-visit-issue)
     ("v p" "pull-request"   forge-visit-pullreq)]
    ["Browse"
     ("b t" "topic"          forge-browse-topic
-     :inapt-if-not forge--get-repository:tracked?)
+     :inapt-if-not (##forge-get-repository :tracked?))
     ("b i" "issue"          forge-browse-issue
-     :inapt-if-not forge--get-repository:tracked?)
+     :inapt-if-not (##forge-get-repository :tracked?))
     ("b p" "pull-request"   forge-browse-pullreq
-     :inapt-if-not forge--get-repository:tracked?)
+     :inapt-if-not (##forge-get-repository :tracked?))
     ("b r" "remote"         forge-browse-remote)
     ("b I" "issues"         forge-browse-issues)
     ("b P" "pull-requests"  forge-browse-pullreqs)
@@ -109,7 +112,7 @@ Takes the pull-request as only argument and must return a directory."
 
 (transient-augment-suffix forge-dispatch
   :transient #'transient--do-replace
-  :inapt-if (lambda () (eq (oref transient--prefix command) 'forge-dispatch))
+  :inapt-if (##eq (oref transient--prefix command) 'forge-dispatch)
   :inapt-face 'forge-suffix-active)
 
 ;;;###autoload(autoload 'forge-configure "forge-commands" nil t)
@@ -127,7 +130,7 @@ Takes the pull-request as only argument and must return a directory."
 
 (transient-augment-suffix forge-configure
   :transient #'transient--do-replace
-  :inapt-if (lambda () (eq (oref transient--prefix command) 'forge-configure))
+  :inapt-if (##eq (oref transient--prefix command) 'forge-configure)
   :inapt-face 'forge-suffix-active)
 
 ;;; Pull
@@ -198,7 +201,7 @@ repository cannot be determined, instead invoke `forge-add-repository'."
 ;;;###autoload(autoload 'forge-pull-topic "forge-commands" nil t)
 (transient-define-suffix forge-pull-topic (number)
   "Read a topic TYPE and NUMBER pull data about it from its forge."
-  :inapt-if-not (lambda () (and (forge--get-repository:tracked?)
+  :inapt-if-not (lambda () (and (forge-get-repository :tracked?)
                            (forge--get-github-repository)))
   (interactive
    (list (read-number "Pull topic: "
@@ -221,6 +224,13 @@ repository cannot be determined, instead invoke `forge-add-repository'."
 
 ;;; Browse
 
+;;;###autoload
+(defun forge-browse-discussions ()
+  "Visit the current repository's discussions using a browser."
+  (interactive)
+  (browse-url (forge--format (forge-get-repository :stub)
+                             'discussions-url-format)))
+
 ;;;###autoload
 (defun forge-browse-issues ()
   "Visit the current repository's issues using a browser."
@@ -243,6 +253,14 @@ also offer closed topics."
   (interactive (list (forge-read-topic "Browse topic")))
   (forge--browse-topic topic))
 
+;;;###autoload
+(defun forge-browse-discussion (discussion)
+  "Read a DISCUSSION and visit it using a browser.
+By default only offer open discussions but with a prefix argument
+also offer closed issues."
+  (interactive (list (forge-read-discussion "Browse discussion")))
+  (forge--browse-topic discussion))
+
 ;;;###autoload
 (defun forge-browse-issue (issue)
   "Read an ISSUE and visit it using a browser.
@@ -293,6 +311,24 @@ argument also offer closed pull-requests."
   (interactive (list (forge-read-repository "Browse repository")))
   (browse-url (forge-get-url repository)))
 
+;;;###autoload
+(defun forge-browse-blob (commit file &optional line end force-hash)
+  "Visit a blob using a browser.
+
+When invoked from a blob- or file-visiting buffer, visit that blob
+without prompting.  If the region is active, try to jump to the marked
+line or lines, and highlight them in the browser.  To what extend that
+is possible depends on the forge.  When the region is not active just
+visit the blob, without trying to jump to the current line.  When
+jumping to a line, always use a commit hash as part of the URL.  From
+a file in the worktree with no active region, instead use the branch
+name as part of the URL, unless a prefix argument is used.
+
+When invoked from any other buffer, prompt the user for a branch or
+commit, and for a file."
+  (interactive (forge--browse-blob-args))
+  (browse-url (forge-get-url :blob commit file line end force-hash)))
+
 ;;;###autoload(autoload 'forge-browse-this-topic "forge-commands" nil t)
 (transient-define-suffix forge-browse-this-topic ()
   "Visit the topic at point using a browser."
@@ -308,7 +344,7 @@ argument also offer closed pull-requests."
 
 ;;;###autoload
 (defun forge-copy-url-at-point-as-kill ()
-  "Copy the url of the thing at point."
+  "Copy the url of thing at point or the thing visited in the current buffer."
   (interactive)
   (if-let ((target (forge--browse-target)))
       (let ((url (if (stringp target) target (forge-get-url target))))
@@ -337,17 +373,40 @@ argument also offer closed pull-requests."
         (forge-get-url :branch branch))
       (and-let* ((remote (magit-remote-at-point)))
         (forge-get-url :remote remote))
+      (and-let* ((file (magit-file-at-point)))
+        (forge-get-url :blob nil file))
       (forge-post-at-point)
       (forge-current-topic)
+      (and (or magit-buffer-file-name buffer-file-name)
+           (apply #'forge-get-url :blob (forge--browse-blob-args)))
       (and magit-buffer-revision
            (forge-get-url :commit magit-buffer-revision))
       (forge-get-repository :stub?)))
 
+(defun forge--browse-blob-args ()
+  (cond
+   (magit-buffer-file-name
+    `(,(or magit-buffer-refname magit-buffer-revision)
+      ,(magit-file-relative-name magit-buffer-file-name)
+      ,@(magit-file-region-line-numbers)
+      ,current-prefix-arg))
+   (buffer-file-name
+    `(nil
+      ,(magit-file-relative-name buffer-file-name)
+      ,@(magit-file-region-line-numbers)
+      ,current-prefix-arg))
+   ((let ((commit (magit-read-local-branch-or-commit
+                   "Browse file from commit")))
+      (list commit (magit-read-file-from-rev commit "Browse file"))))))
+
 ;;;; Urls
 
 (cl-defgeneric forge-get-url (obj)
   "Return the URL for a forge object.")
 
+(cl-defmethod forge-get-url ((disc forge-discussion))
+  (forge--format disc 'discussion-url-format))
+
 (cl-defmethod forge-get-url ((issue forge-issue))
   (forge--format issue 'issue-url-format))
 
@@ -369,12 +428,27 @@ argument also offer closed pull-requests."
     (forge--format repo 'commit-url-format
                    `((?r . ,(magit-commit-p commit))))))
 
+(cl-defmethod forge-get-url ((_(eql :blob)) commit file
+                             &optional line end force-hash)
+  (let* ((commit (or (and (magit-branch-p commit)
+                          (cdr (magit-split-branch-name commit)))
+                     (and commit (magit-commit-p commit))
+                     (and (not (or line force-hash))
+                          (magit-get-current-branch))
+                     (magit-rev-parse "HEAD")))
+         (repo   (forge-get-repository :stub))
+         (format (oref repo blob-url-format)))
+    (when (cl-typep repo 'forge-gitweb-repository)
+      (setq commit (concat (if (magit-branch-p commit) "hb=" "h=") commit)))
+    (concat
+     (forge--format repo format `((?r . ,commit) (?f . ,file)))
+     (and line (forge-format-blob-lines repo line
+                                        (and (not (equal line end)) end))))))
+
 (cl-defmethod forge-get-url ((_(eql :branch)) branch)
   (let (remote)
     (if (magit-remote-branch-p branch)
-        (let ((cons (magit-split-branch-name branch)))
-          (setq remote (car cons))
-          (setq branch (cdr cons)))
+        (pcase-setq `(,remote . ,branch) (magit-split-branch-name branch))
       (unless (setq remote (or (magit-get-push-remote branch)
                                (magit-get-upstream-remote branch)))
         (user-error "Cannot determine remote for %s" branch)))
@@ -387,7 +461,9 @@ argument also offer closed pull-requests."
 
 (cl-defmethod forge-get-url ((post forge-post))
   (forge--format post (let ((topic (forge-get-parent post)))
-                        (cond ((forge--childp topic 'forge-issue)
+                        (cond ((forge--childp topic 'forge-discussion)
+                               'discussion-post-url-format)
+                              ((forge--childp topic 'forge-issue)
                                'issue-post-url-format)
                               ((forge--childp topic 'forge-pullreq)
                                'pullreq-post-url-format)))))
@@ -395,6 +471,30 @@ argument also offer closed pull-requests."
 (cl-defmethod forge-get-url ((notify forge-notification))
   (oref notify url))
 
+ ;; Transitional kludge for the infamous package.el defect.
+(require 'forge-forgejo)
+
+(cl-defmethod forge-format-blob-lines ((repo forge-repository) line end)
+  (cl-etypecase repo ;Third-party classes require separate methods.
+    ((or forge-github-repository
+         forge-gitlab-repository ;Also supports "#L%s-%s".
+         forge-forgejo-repository
+         forge-gitea-repository
+         forge-gogs-repository)
+     (format (if end "#L%s-L%s" "#L%s") line end))
+    (forge-bitbucket-repository
+     (format (if end "#lines-%s:%s" "#lines-%s") line end))
+    ((or forge-cgit-repository
+         forge-cgit*-repository
+         forge-cgit**-repository)
+     (format "#n%s" line))
+    ((or forge-gitweb-repository
+         forge-repoorcz-repository
+         forge-stagit-repository)
+     (format "#l%s" line))
+    (forge-srht-repository
+     (format "#L%s" line))))
+
 ;;; Visit
 
 ;;;###autoload
@@ -407,6 +507,16 @@ the limitation to active topics."
   (interactive (list (forge-read-topic "View topic")))
   (forge-topic-setup-buffer (forge-get-topic topic)))
 
+;;;###autoload
+(defun forge-visit-discussion (discussion)
+  "Read a DISCUSSION and visit it.
+By default only offer active topics for completion.  With a prefix
+argument offer all topics.  While completion is in progress, \
+\\<forge-read-topic-minibuffer-map>\\[forge-read-topic-lift-limit] lifts
+the limitation to active topics."
+  (interactive (list (forge-read-discussion "View discussion")))
+  (forge-topic-setup-buffer (forge-get-discussion discussion)))
+
 ;;;###autoload
 (defun forge-visit-issue (issue)
   "Read an ISSUE and visit it.
@@ -462,6 +572,21 @@ With prefix argument MENU, also show the topic menu."
 
 ;;; Create
 
+(defun forge-create-discussion (category)
+  "Create a new discussion for the current repository."
+  (interactive
+   (list (forge-read-topic-category nil "Category for new discussion")))
+  (let* ((repo (forge-get-repository :tracked))
+         (buf (forge--prepare-post-buffer
+               "new-discussion"
+               (forge--format repo "Create new discussion on %p"))))
+    (when buf
+      (with-current-buffer buf
+        (setq forge--buffer-post-object repo)
+        (setq forge--submit-post-function
+              (rpartial #'forge--submit-create-discussion category)))
+      (forge--display-post-buffer buf))))
+
 (defun forge-create-issue ()
   "Create a new issue for the current repository."
   (interactive)
@@ -506,7 +631,10 @@ With prefix argument MENU, also show the topic menu."
                                     issue source target))
 
 (defun forge-create-pullreq--read-args ()
-  (let* ((source  (magit-completing-read
+  (let* ((repo (forge-get-repository :tracked))
+         (_ (unless (oref repo worktree)
+              (user-error "Cannot create pull-request without working tree")))
+         (source  (magit-completing-read
                    "Source branch"
                    (magit-list-remote-branch-names)
                    nil t nil 'magit-revision-history
@@ -518,7 +646,6 @@ With prefix argument MENU, also show the topic menu."
                          (if (magit-remote-branch-p d)
                              d
                            (magit-get-push-branch d t))))))
-         (repo    (forge-get-repository :tracked))
          (remote  (oref repo remote))
          (targets (delete source (magit-list-remote-branch-names remote)))
          (target  (magit-completing-read
@@ -543,11 +670,7 @@ point is currently on."
   (interactive (list current-prefix-arg))
   (unless (derived-mode-p 'forge-topic-mode)
     (user-error "This command is only available from topic buffers"))
-  (let* ((topic forge-buffer-topic)
-         (buf (forge--prepare-post-buffer
-               (forge--format topic "%i;new-comment")
-               (forge--format topic "New comment on #%i of %p")))
-         (quote (cond
+  (let* ((quote (cond
                  ((not (magit-section-match 'post)) nil)
                  ((use-region-p)
                   (buffer-substring-no-properties (region-beginning)
@@ -556,9 +679,25 @@ point is currently on."
                   (let ((section (magit-current-section)))
                     (string-trim-right
                      (buffer-substring-no-properties (oref section content)
-                                                     (oref section end))))))))
+                                                     (oref section end)))))))
+         (obj (if (forge-discussion-p forge-buffer-topic)
+                  (forge--select-discussion-reply-target)
+                forge-buffer-topic))
+         (buf (cond
+               ((forge-discussion-post-p obj)
+                (forge--prepare-post-buffer
+                 (forge--format obj "%i;%I;new-reply")
+                 (forge--format obj "New comment on #%i;%I of %p")))
+               ((forge-discussion-p obj)
+                (forge--prepare-post-buffer
+                 (forge--format obj "%i;new-answer")
+                 (forge--format obj "New comment on #%i of %p")))
+               (t
+                (forge--prepare-post-buffer
+                 (forge--format obj "%i;new-comment")
+                 (forge--format obj "New comment on #%i of %p"))))))
     (with-current-buffer buf
-      (setq forge--buffer-post-object topic)
+      (setq forge--buffer-post-object obj)
       (setq forge--submit-post-function #'forge--submit-create-post)
       (when quote
         (goto-char (point-max))
@@ -567,6 +706,41 @@ point is currently on."
         (insert (replace-regexp-in-string "^" "> " quote) "\n\n")))
     (forge--display-post-buffer buf)))
 
+(transient-define-suffix forge-approve-pullreq ()
+  "Approve the current pull-request."
+  :description "approve pull-request"
+  :inapt-if-not #'forge-current-pullreq
+  :transient nil
+  (interactive)
+  (let ((pullreq (forge-current-pullreq t)))
+    (unless (cl-typep (forge-get-repository pullreq) 'forge-github-repository)
+      (user-error "This command is only available for Github"))
+    (when-let ((buf (forge--prepare-post-buffer
+                     (forge--format pullreq "%i;new-approval")
+                     (forge--format pullreq "Approve pull-request #%i of %p"))))
+      (with-current-buffer buf
+        (setq forge--buffer-post-object pullreq)
+        (setq forge--submit-post-function #'forge--submit-approve-pullreq))
+      (forge--display-post-buffer buf))))
+
+(transient-define-suffix forge-request-changes ()
+  "Request changes to the current pull-request."
+  :description "request changes"
+  :inapt-if-not #'forge-current-pullreq
+  :transient nil
+  (interactive)
+  (let ((pullreq (forge-current-pullreq t)))
+    (unless (cl-typep (forge-get-repository pullreq) 'forge-github-repository)
+      (user-error "This command is only available for Github"))
+    (when-let ((buf (forge--prepare-post-buffer
+                     (forge--format pullreq "%i;new-request")
+                     (forge--format
+                      pullreq "Request changes for pull-request #%i of %p"))))
+      (with-current-buffer buf
+        (setq forge--buffer-post-object pullreq)
+        (setq forge--submit-post-function #'forge--submit-request-changes))
+      (forge--display-post-buffer buf))))
+
 ;;; Edit
 
 (defun forge-edit-post ()
@@ -594,6 +768,7 @@ point is currently on."
 (transient-define-suffix forge-edit-topic-note ()
   "Edit your private note about the current topic."
   :transient #'transient--do-quit-all
+  :inapt-if-not #'forge-current-topic
   :description
   (lambda ()
     (if-let ((topic (forge-current-topic)))
@@ -789,7 +964,7 @@ configure it first."
 (defun forge-checkout-worktree (path pullreq)
   "Create, configure and checkout a new worktree from a pull-request.
 This is like `forge-checkout-pullreq', except that it also
-creates a new worktree. Please see the manual for more
+creates a new worktree.  Please see the manual for more
 information."
   (interactive
    (let ((id (forge-read-pullreq "Checkout pull request")))
@@ -919,12 +1094,41 @@ as merged."
   (forge-pull))
 
 ;;;###autoload
-(defun forge-rename-default-branch ()
-  "Rename the default branch to NEWNAME.
-Change the name on the upstream remotely and locally, and update the
-upstream remotes of local branches accordingly."
+(defun forge-set-default-branch ()
+  "Change the default branch on the upstream remote and locally.
+Also update the upstream branches of local branches accordingly."
   (interactive)
-  (let* ((repo (forge-get-repository :tracked?))
+  (pcase-let* ((`(,repo ,old) (forge--set-default-branch-read-args))
+               (new (magit-read-remote-branch
+                     (format "Set default branch (was: %s)" old)
+                     (delete old (forge--get-remote))
+                     nil nil t)))
+    (message "Changing default branch...")
+    (forge--set-default-branch repo new)
+    (magit--set-default-branch new old)
+    (forge-refresh-buffer)
+    (message "Changing default branch...done")))
+
+;;;###autoload
+(defun forge-rename-default-branch ()
+  "Rename the default branch on the upstream remote and locally.
+Also update the upstream branches of local branches accordingly."
+  (interactive)
+  (pcase-let* ((`(,repo ,old) (forge--set-default-branch-read-args))
+               (default (and (not (equal old "main")) "main"))
+               (new (read-string
+                     (format "Rename default branch `%s' to%s: " old
+                             (if default (format " (default: %s)" default) ""))
+                     nil nil default)))
+    (message "Renaming default branch...")
+    (forge--rename-branch repo new old)
+    (magit--set-default-branch new old)
+    (magit-git "remote" "set-head" "--auto" (oref repo remote))
+    (forge-refresh-buffer)
+    (message "Renaming default branch...done")))
+
+(defun forge--set-default-branch-read-args ()
+  (let* ((repo (forge-get-repository :tracked))
          (_ (unless (forge-github-repository-p repo)
               (user-error "Updating default branch not supported for forge `%s'"
                           (oref repo forge))))
@@ -938,20 +1142,11 @@ upstream remotes of local branches accordingly."
                     (magit-git "fetch" "--prune")
                     (magit-git "remote" "set-head" "--auto" remote)
                     (message "Determining old default branch...done")
-                    (magit-git-string "symbolic-ref" "--short" symref)))
-         (oldname (if oldhead
-                      (cdr (magit-split-branch-name oldhead))
-                    (error "Cannot determine old default branch")))
-         (default (and (not (equal oldname "main")) "main"))
-         (newname (read-string
-                   (format "Rename default branch `%s' to%s: "
-                           oldname
-                           (if default (format " (default: %s)" default) ""))
-                   nil nil default)))
-    (message "Renaming default branch...")
-    (forge--set-default-branch repo newname oldname)
-    (forge-refresh-buffer)
-    (message "Renaming default branch...done")))
+                    (magit-git-string "symbolic-ref" "--short" symref))))
+    (list repo
+          (if oldhead
+              (cdr (magit-split-branch-name oldhead))
+            (error "Cannot determine old default branch")))))
 
 ;;; Configuration
 
@@ -977,7 +1172,7 @@ upstream remotes of local branches accordingly."
   :class 'magit--git-variable
   :variable "forge.graphqlItemLimit"
   :reader #'read-string
-  :default (lambda () (number-to-string ghub-graphql-items-per-request)))
+  :default (##number-to-string ghub-graphql-items-per-request))
 
 (transient-define-suffix forge-toggle-display-in-status-buffer ()
   "Toggle whether to display topics in the current status buffer."
@@ -1034,7 +1229,7 @@ upstream remote."
   [:class transient-subgroups
 
    ;; Already tracked.
-   [:if (lambda () (forge--scope :tracked))
+   [:if (##forge--scope :tracked)
     (:info*
      (lambda ()
        (format
@@ -1043,7 +1238,7 @@ upstream remote."
      :format "%d")]
 
    ;; Nothing to tracked.
-   [:if-not (lambda () (forge--scope 'topdir))
+   [:if-not (##forge--scope 'topdir)
     (:info*
      (lambda ()
        (format
@@ -1052,7 +1247,7 @@ upstream remote."
      :format "%d")]
 
    ;; Cannot track.
-   [:if (lambda () (and (not (forge--scope 'repo)) (forge--scope 'topdir)))
+   [:if (##and (not (forge--scope 'repo)) (forge--scope 'topdir))
     :description
     (lambda ()
       (concat
@@ -1075,7 +1270,7 @@ upstream remote."
      (lambda () (interactive) (info "(forge)Setup a Partially Supported Host")))]
 
    ;; Track it!
-   [:if (lambda () (forge--scope :untracked))
+   [:if (##forge--scope :untracked)
     :description
     (lambda ()
       (format
@@ -1214,18 +1409,35 @@ This may take a while.  Only Github is supported at the moment."
 ;;;###autoload
 (defun forge-remove-topic-locally (topic)
   "Remove a topic from the local database only.
-Due to how the supported APIs work, it would be too expensive to
-automatically remove topics from the local database that were
-removed from the forge.  The purpose of this command is to allow
-you to manually clean up the local database."
-  (interactive (list (forge-read-topic "Delete topic LOCALLY only")))
-  (setq topic (forge-get-topic topic))
-  (closql-delete topic)
-  (if (and (derived-mode-p 'forge-topic-mode)
-           (eq (oref topic id)
-               (oref forge-buffer-topic id)))
-      (kill-buffer (current-buffer))
-    (forge-refresh-buffer)))
+
+When the region marks multiple topics, then offer to remove them all.
+
+The topic is not removed from the forge and, if it is later modified,
+then it will be added to the database again when fetching all topics.
+
+This is useful for users who only fetch individual topics and want to
+remove the topics they are no longer interested in.  This can also be
+used to remove topics locally, which have already been removed on the
+forge (the service).  Forge (the package) cannot automatically detect
+when that happens, because given how the APIs work, this would be too
+expensive."
+  (interactive
+   (list (if-let* ((topics (magit-region-values '(issue pullreq) t))
+                   ((magit-confirm 'remove-topics-locally nil
+                      "Delete %d topics locally" nil
+                      (mapcar #'forge--format-topic-line topics))))
+             topics
+           (forge-read-topic "Delete topic LOCALLY only"))))
+  (if (listp topic)
+      (progn (mapc #'closql-delete topic)
+             (forge-refresh-buffer))
+    (setq topic (forge-get-topic topic))
+    (closql-delete topic)
+    (if (and (derived-mode-p 'forge-topic-mode)
+             (equal (oref topic id)
+                    (oref forge-buffer-topic id)))
+        (kill-buffer (current-buffer))
+      (forge-refresh-buffer))))
 
 ;;;###autoload
 (defun forge-reset-database ()
@@ -1243,9 +1455,41 @@ heavy development."
 
 ;;; Miscellaneous
 
+(defun forge-mark-completed-topics-as-done ()
+  "Mark completed topics of the current repository as done.
+Change the private status to \"done\" for topics whose private status is
+\"unread\" or \"pending\" and whose public state is \"completed\".
+Whether this affects all such topics or only all such topics of a
+certain type (discussion, issue or pull-request), depends on the
+context."
+  (interactive)
+  (let* ((type (forge-current-topic-type))
+         (desc (if (eq type 'pullreq) 'pull-request type))
+         (topics (forge--list-topics
+                  (forge--topics-spec :type type
+                                      :active nil
+                                      :state 'closed
+                                      :status 'inbox)
+                  (forge-get-repository :tracked))))
+    (cond ((not topics)
+           (message "No completed %s that could be marked as done" desc))
+          ((magit-confirm t
+             "Mark \"%s\" as done"
+             (format "Mark %%d %ss as done" desc)
+             nil
+             (mapcar #'forge--format-topic-line topics))
+           (dolist (topic topics)
+             (oset topic status 'done))
+           (forge-refresh-buffer)))))
+
 (magit-define-section-jumper forge-jump-to-pullreqs "Pull requests" pullreqs)
 (magit-define-section-jumper forge-jump-to-issues "Issues" issues)
 
 ;;; _
+;; Local Variables:
+;; read-symbol-shorthands: (
+;;   ("partial" . "llama--left-apply-partially")
+;;   ("rpartial" . "llama--right-apply-partially"))
+;; End:
 (provide 'forge-commands)
 ;;; forge-commands.el ends here
