@@ -1,6 +1,6 @@
 ;;; forge-notify.el --- Notify support  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2018-2024 Jonas Bernoulli
+;; Copyright (C) 2018-2025 Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
 ;; Maintainer: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
@@ -93,7 +93,7 @@ signal an error."
          (savedp (memq 'saved status))
          (status (remq 'saved status)))
     (mapcar
-     (lambda (row) (closql--remake-instance 'forge-notification (forge-db) row))
+     (partial #'closql--remake-instance 'forge-notification (forge-db))
      (if (seq-set-equal-p status '(unread pending done) #'eq)
          (forge-sql [:select * :from notification :order-by [(desc updated)]])
        (forge-sql
@@ -177,7 +177,7 @@ signal an error."
 (transient-augment-suffix forge-notifications-menu
   :transient #'transient--do-replace
   :if-mode 'forge-notifications-mode
-  :inapt-if (lambda () (eq (oref transient--prefix command) 'forge-notifications-menu))
+  :inapt-if (##eq (oref transient--prefix command) 'forge-notifications-menu)
   :inapt-face 'forge-suffix-active)
 
 ;;;###autoload(autoload 'forge-list-notifications "forge-notify" nil t)
@@ -193,7 +193,7 @@ signal an error."
 (transient-define-suffix forge-notifications-display-inbox ()
   "List unread and pending notifications."
   :description "inbox"
-  :inapt-if (lambda () (equal forge-notifications-selection '(unread pending)))
+  :inapt-if (##equal forge-notifications-selection '(unread pending))
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -204,7 +204,7 @@ signal an error."
 (transient-define-suffix forge-notifications-display-saved ()
   "List saved notifications."
   :description "saved"
-  :inapt-if (lambda () (eq forge-notifications-selection 'saved))
+  :inapt-if (##eq forge-notifications-selection 'saved)
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -215,7 +215,7 @@ signal an error."
 (transient-define-suffix forge-notifications-display-done ()
   "List done notifications."
   :description "done"
-  :inapt-if (lambda () (eq forge-notifications-selection 'done))
+  :inapt-if (##eq forge-notifications-selection 'done)
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -226,7 +226,7 @@ signal an error."
 (transient-define-suffix forge-notifications-display-all ()
   "List all notifications."
   :description "all"
-  :inapt-if (lambda () (equal forge-notifications-selection '(unread pending done)))
+  :inapt-if (##equal forge-notifications-selection '(unread pending done))
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -237,7 +237,7 @@ signal an error."
 (transient-define-suffix forge-notifications-style-flat ()
   "Show a flat notification list."
   :description "single list"
-  :inapt-if (lambda () (eq forge-notifications-display-style 'flat))
+  :inapt-if (##eq forge-notifications-display-style 'flat)
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -248,7 +248,7 @@ signal an error."
 (transient-define-suffix forge-notifications-style-nested ()
   "Group notifications by repository."
   :description "group by repo"
-  :inapt-if (lambda () (eq forge-notifications-display-style 'nested))
+  :inapt-if (##eq forge-notifications-display-style 'nested)
   :inapt-face 'forge-suffix-active
   (interactive)
   (unless (derived-mode-p 'forge-notifications-mode)
@@ -281,7 +281,7 @@ signal an error."
             (forge-insert-notification notif))
           (insert ?\n)))
        ((pcase-dolist (`(,_ . ,notifs)
-                       (--group-by (oref it repository) notifs))
+                       (seq-group-by (##oref % repository) notifs))
           (let ((repo (forge-get-repository (car notifs))))
             (magit-insert-section (forge-repo repo)
               (magit-insert-heading
@@ -295,7 +295,7 @@ signal an error."
 (defun forge-insert-notification (notif)
   (with-slots (type title url) notif
     (pcase type
-      ((or 'issue 'pullreq)
+      ((or 'discussion 'issue 'pullreq)
        (forge--insert-topic (forge-get-topic notif)))
       ('commit
        (magit-insert-section (ncommit nil) ; !commit
@@ -305,11 +305,10 @@ signal an error."
                   (propertize (substring (match-string 0 url)
                                          0 (magit-abbrev-length))
                               'font-lock-face 'magit-hash)
-                  (magit-log-propertize-keywords
-                   nil
+                  (magit-log--wash-summary
                    (propertize title 'font-lock-face
-                               (if-let ((topic (oref notif topic))
-                                        ((eq (oref topic status) 'unread)))
+                               (if-let* ((topic (oref notif topic))
+                                         ((eq (oref topic status) 'unread)))
                                    'forge-topic-unread
                                  'forge-topic-open)))))))
       (_
@@ -321,5 +320,10 @@ signal an error."
                              'font-lock-face 'error)))))))
 
 ;;; _
+;; Local Variables:
+;; read-symbol-shorthands: (
+;;   ("partial" . "llama--left-apply-partially")
+;;   ("rpartial" . "llama--right-apply-partially"))
+;; End:
 (provide 'forge-notify)
 ;;; forge-notify.el ends here
