@@ -1,6 +1,6 @@
 ;;; forge-topics.el --- List topics  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2018-2024 Jonas Bernoulli
+;; Copyright (C) 2018-2025 Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
 ;; Maintainer: Jonas Bernoulli <emacs.forge@jonas.bernoulli.dev>
@@ -133,6 +133,7 @@ Must be set before `forge-topics' is loaded.")
       (eieio-oset spec key (pop params)))
     (unless (oref spec type)
       (oset spec type 'topic))
+    (forge--cast-topics-spec-state spec)
     (unless (or repo global)
       (error "Cannot determine repository"))
     (magit-setup-buffer-internal #'forge-topics-mode nil
@@ -156,7 +157,7 @@ Must be set before `forge-topics' is loaded.")
         (dolist (topic topics)
           (forge--insert-topic topic 5)))
        ((pcase-dolist (`(,_ . ,topics)
-                       (--group-by (oref it repository) topics))
+                       (seq-group-by (##oref % repository) topics))
           (let ((repo (forge-get-repository (car topics))))
             (magit-insert-section (forge-repo repo)
               (magit-insert-heading
@@ -193,8 +194,11 @@ Must be set before `forge-topics' is loaded.")
    ["State"
     ("a" forge-topics-filter-active)
     ("o" forge-topics-filter-state-open)
-    ("c" forge-topics-filter-state-completed)
-    ("x" forge-topics-filter-state-unplanned)]
+    ("r" forge-topics-filter-state-realized)
+    ("e" forge-topics-filter-state-expunged)
+    ("U" forge-topics-filter-state-unplanned)
+    ("O" forge-topics-filter-state-outdated)
+    ("D" forge-topics-filter-state-duplicate)]
    ["Status"
     ("i" forge-topics-filter-status-inbox)
     ("u" forge-topics-filter-status-unread)
@@ -202,14 +206,16 @@ Must be set before `forge-topics' is loaded.")
     ("d" forge-topics-filter-status-done)]
    ["Type"
     ("t t" forge-topics-all-types)
+    ("t d" forge-topics-filter-discussions)
     ("t i" forge-topics-filter-issues)
     ("t p" forge-topics-filter-pullreqs)]]
   [forge--lists-group
    ["Filter                                      "
+    ("-c" forge-topics-filter-category)
     ("-m" forge-topics-filter-milestone)
     ("-l" forge-topics-filter-labels)
     ("-x" forge-topics-filter-marks)
-    ("-c" forge-topics-filter-author)
+    ("-A" forge-topics-filter-author)
     ("-a" forge-topics-filter-assignee)
     ("-r" forge-topics-filter-reviewer)
     ("-s" forge-topics-filter-saved)]
@@ -232,7 +238,7 @@ Must be set before `forge-topics' is loaded.")
 (transient-augment-suffix forge-topics-menu
   :transient #'transient--do-replace
   :if-not-derived '(forge-notifications-mode forge-repository-list-mode)
-  :inapt-if (lambda () (eq (oref transient--prefix command) 'forge-topics-menu))
+  :inapt-if (##eq (oref transient--prefix command) 'forge-topics-menu)
   :inapt-face 'forge-suffix-active)
 
 (defvar-local forge--quit-keep-topic-menu nil)
@@ -275,10 +281,10 @@ then display the respective menu, otherwise display no menu."
 (transient-define-suffix forge-list-topics (&optional repo)
   "List topics of the current repository."
   :description "topics"
-  :inapt-if (lambda () (or (not (forge--get-repository:tracked?))
+  :inapt-if (lambda () (or (not (forge-get-repository :tracked?))
                       (and (eq major-mode 'forge-topics-mode)
                            (not (oref forge--buffer-topics-spec global)))))
-  :inapt-face (lambda () (if (not (forge--get-repository:tracked?))
+  :inapt-face (lambda () (if (not (forge-get-repository :tracked?))
                         'transient-inapt-suffix
                       'forge-suffix-active))
   (declare (interactive-only nil))
@@ -286,6 +292,15 @@ then display the respective menu, otherwise display no menu."
   (forge-topics-setup-buffer repo)
   (transient-setup 'forge-topics-menu))
 
+;;;###autoload(autoload 'forge-list-discussions "forge-topics" nil t)
+(transient-define-suffix forge-list-discussions (&optional repo)
+  "List discussions of the current repository."
+  :description "discussions"
+  (declare (interactive-only nil))
+  (interactive)
+  (forge-topics-setup-buffer repo nil :type 'discussion)
+  (transient-setup 'forge-topics-menu))
+
 ;;;###autoload(autoload 'forge-list-issues "forge-topics" nil t)
 (transient-define-suffix forge-list-issues (&optional repo)
   "List issues of the current repository."
@@ -343,6 +358,7 @@ then display the respective menu, otherwise display no menu."
                 (interactive)
                 (oset forge--buffer-topics-spec type
                       (oref (transient-suffix-object) type))
+                (forge--cast-topics-spec-state forge--buffer-topics-spec)
                 (forge-refresh-buffer)))
    (inapt-face :initform 'forge-suffix-active)
    (inapt-if
@@ -354,6 +370,11 @@ then display the respective menu, otherwise display no menu."
   :class 'forge--topics-filter-type-command :type 'topic
   :description "topics")
 
+(transient-define-suffix forge-topics-filter-discussions ()
+  "List discussions of the current repository."
+  :class 'forge--topics-filter-type-command :type 'discussion
+  :description "discussions")
+
 (transient-define-suffix forge-topics-filter-issues ()
   "List issues of the current repository."
   :class 'forge--topics-filter-type-command :type 'issue
@@ -369,9 +390,7 @@ then display the respective menu, otherwise display no menu."
 (transient-define-suffix forge-topics-filter-active ()
   "Limit topic list to active topics."
   :description "active"
-  :face (lambda ()
-          (and (oref forge--buffer-topics-spec active)
-               'forge-suffix-active))
+  :face (##and (oref forge--buffer-topics-spec active) 'forge-suffix-active)
   (interactive)
   (oset forge--buffer-topics-spec active
         (not (oref forge--buffer-topics-spec active)))
@@ -398,14 +417,7 @@ then display the respective menu, otherwise display no menu."
                 (forge-refresh-buffer)))
    (description
     :initform (lambda (suffix)
-                (let ((want (oref suffix state))
-                      (type (oref forge--buffer-topics-spec type)))
-                  (pcase type
-                    ((guard (atom want))
-                     (symbol-name want))
-                    ('topic   (apply #'format "%s/%s" want))
-                    ('issue   (symbol-name (car want)))
-                    ('pullreq (symbol-name (cadr want)))))))
+                (symbol-name (oref suffix state))))
    (face
     :initform (lambda (suffix)
                 (let ((want   (oref suffix state))
@@ -419,19 +431,70 @@ then display the respective menu, otherwise display no menu."
                               (eq want 'open))
                          (if (eq have want)
                              'forge-suffix-active-and-implied
-                           'forge-suffix-implied))))))))
+                           'forge-suffix-implied))
+                        ((and (memq want '(unplanned duplicate outdated))
+                              (equal have
+                                     '(unplanned duplicate outdated rejected))
+                              (not active))
+                         'forge-suffix-implied)))))))
 
 (transient-define-suffix forge-topics-filter-state-open ()
   "Limit topic list to open topics."
-  :class 'forge--topics-filter-state-command :state 'open)
+  :class 'forge--topics-filter-state-command
+  :state 'open)
 
-(transient-define-suffix forge-topics-filter-state-completed ()
-  "Limit topic list to completed and merged topics."
-  :class 'forge--topics-filter-state-command :state '(completed merged))
+(transient-define-suffix forge-topics-filter-state-realized ()
+  "Limit topic list to realized topics.
+Realized topics include:
+- completed discussions,
+- completed issues, and
+- merged pull-requests."
+  :class 'forge--topics-filter-state-command
+  :state '(completed merged)
+  :description (lambda ()
+                 (pcase (oref forge--buffer-topics-spec type)
+                   ('discussion "completed")
+                   ('issue      "completed")
+                   ('pullreq    "merged")
+                   ('topic      "realized"))))
+
+(transient-define-suffix forge-topics-filter-state-expunged ()
+  "Limit topic list to expunged topics.
+Expunged topics include:
+- discussions closed as outdated,
+- discussions closed as duplicates,
+- issues closed as unplanned,
+- issues closed as duplicates, and
+- pull-requests closed without merging."
+  :class 'forge--topics-filter-state-command
+  :state '(unplanned duplicate outdated rejected)
+  :description (lambda ()
+                 (pcase (oref forge--buffer-topics-spec type)
+                   ('discussion "expunged")
+                   ('issue      "expunged")
+                   ('pullreq    "rejected")
+                   ('topic      "expunged"))))
 
 (transient-define-suffix forge-topics-filter-state-unplanned ()
-  "Limit topic list to unplanned and rejected topics."
-  :class 'forge--topics-filter-state-command :state '(unplanned rejected))
+  "Limit topic list to issues closed as unplanned."
+  :class 'forge--topics-filter-state-command
+  :state 'unplanned
+  :description "  unplanned"
+  :if (##eq (oref forge--buffer-topics-spec type) 'issue))
+
+(transient-define-suffix forge-topics-filter-state-outdated ()
+  "Limit topic list to discussions closed as outdated."
+  :class 'forge--topics-filter-state-command
+  :state 'outdated
+  :description "  outdated"
+  :if (##eq (oref forge--buffer-topics-spec type) 'discussion))
+
+(transient-define-suffix forge-topics-filter-state-duplicate ()
+  "Limit topic list to discussions and issues closed as duplicates."
+  :class 'forge--topics-filter-state-command
+  :state 'duplicate
+  :description "  duplicate"
+  :if (##memq (oref forge--buffer-topics-spec type) '(discussion issue)))
 
 ;;;; Status
 
@@ -516,23 +579,29 @@ then display the respective menu, otherwise display no menu."
   (unless (slot-boundp obj 'reader)
     (oset obj reader (intern (format "forge-read-topic-%s" (oref obj slot))))))
 
+(transient-define-suffix forge-topics-filter-category ()
+  "Read a category and limit discussions to that category."
+  :class 'forge--topics-filter-command
+  :slot 'category
+  :formatter (##propertize % 'face 'forge-topic-label))
+
 (transient-define-suffix forge-topics-filter-milestone ()
   "Read a milestone and limit topic list to topics with that milestone."
   :class 'forge--topics-filter-command
   :slot 'milestone
-  :formatter (lambda (m) (propertize m 'face 'forge-topic-label)))
+  :formatter (##propertize % 'face 'forge-topic-label))
 
 (transient-define-suffix forge-topics-filter-labels ()
   "Read labels and limit topic list to topics with one of these labels."
   :class 'forge--topics-filter-command
   :slot 'labels
-  :formatter (lambda (labels) (and labels (forge--format-labels labels " "))))
+  :formatter (##and % (forge--format-labels % " ")))
 
 (transient-define-suffix forge-topics-filter-marks ()
   "Read marks and limit topic list to topics with one of these marks."
   :class 'forge--topics-filter-command
   :slot 'marks
-  :formatter (lambda (marks) (and marks (forge--format-marks marks " "))))
+  :formatter (##and % (forge--format-marks % " ")))
 
 (transient-define-suffix forge-topics-filter-saved ()
   "Toggle whether to limit topic list to saved topics."
@@ -540,25 +609,25 @@ then display the respective menu, otherwise display no menu."
   :slot 'saved
   :reader #'always
   :description
-  (lambda () (forge--format-boolean 'saved "saved" forge--buffer-topics-spec)))
+  (##forge--format-boolean 'saved "saved" forge--buffer-topics-spec))
 
 (transient-define-suffix forge-topics-filter-author ()
   "Read an author and limit topic list to topics created by that author."
   :class 'forge--topics-filter-command
   :slot 'author
-  :reader (lambda () (forge--read-filter-by-user "Author")))
+  :reader (##forge--read-filter-by-user "Author"))
 
 (transient-define-suffix forge-topics-filter-assignee ()
   "Read an assignee and limit topic list to topics assignee to that person."
   :class 'forge--topics-filter-command
   :slot 'assignee
-  :reader (lambda () (forge--read-filter-by-user "Assignee")))
+  :reader (##forge--read-filter-by-user "Assignee"))
 
 (transient-define-suffix forge-topics-filter-reviewer ()
   "Read a reviewer and limit topic list to reviews requested from that person."
   :class 'forge--topics-filter-command
   :slot 'reviewer
-  :reader (lambda () (forge--read-filter-by-user "Reviewer")))
+  :reader (##forge--read-filter-by-user "Reviewer"))
 
 (defun forge--read-filter-by-user (prompt)
   (let* ((repo (forge-get-repository :tracked))
@@ -578,8 +647,8 @@ then display the respective menu, otherwise display no menu."
    (list (magit-read-char-case "Order by: " t
            (?n "[n]ewest"            'newest)
            (?o "[o]ldest"            'oldest)
-           (?u "[r]ecently updated"  'recently-updated)
-           (?U "[a]nciently updated" 'anciently-updated))))
+           (?r "[r]ecently updated"  'recently-updated)
+           (?a "[a]nciently updated" 'anciently-updated))))
   (oset forge--buffer-topics-spec order order)
   (forge-refresh-buffer))
 
@@ -597,8 +666,8 @@ then display the respective menu, otherwise display no menu."
 (transient-define-suffix forge-topics-group ()
   "Group topics by repository."
   :description "group by repo"
-  :if (lambda () (oref forge--buffer-topics-spec global))
-  :inapt-if (lambda () (oref forge--buffer-topics-spec grouped))
+  :if (##oref forge--buffer-topics-spec global)
+  :inapt-if (##oref forge--buffer-topics-spec grouped)
   :inapt-face 'forge-suffix-active
   (interactive)
   (oset forge--buffer-topics-spec grouped t)
@@ -607,13 +676,18 @@ then display the respective menu, otherwise display no menu."
 (transient-define-suffix forge-topics-ungroup ()
   "Show a flat topic list."
   :description "single list"
-  :if (lambda () (oref forge--buffer-topics-spec global))
-  :inapt-if-not (lambda () (oref forge--buffer-topics-spec grouped))
+  :if (##oref forge--buffer-topics-spec global)
+  :inapt-if-not (##oref forge--buffer-topics-spec grouped)
   :inapt-face 'forge-suffix-active
   (interactive)
   (oset forge--buffer-topics-spec grouped nil)
   (forge-refresh-buffer))
 
 ;;; _
+;; Local Variables:
+;; read-symbol-shorthands: (
+;;   ("partial" . "llama--left-apply-partially")
+;;   ("rpartial" . "llama--right-apply-partially"))
+;; End:
 (provide 'forge-topics)
 ;;; forge-topics.el ends here
